local io = require("io")
local os = require('os')
local rules = require("rules")
local awful = require("awful")
local beautiful = require("beautiful")
local util = require("util")


-- {{{ Link titlebars with floating property

function set_titlebar(c)
	if c.floating and not c.skip_taskbar then
		awful.titlebar.show(c)
	else
		awful.titlebar.hide(c)
	end
end

client.connect_signal("property::floating", set_titlebar)
client.connect_signal("request::titlebars", set_titlebar)
--
-- }}}


-- {{{ Load local configs

local local_dir = '.local/etc/awesome/config'
local status, files = pcall(function() return io.popen('ls '..local_dir..'/*.lua') end)
if status then
	for name in files:lines() do
		dofile(name)
	end
end

-- }}}


-- {{{ Better menus

local main_menu = {
	{ "lock screen", "dm-tool lock" },
	{ "awesome", {
		{ "restart", awesome.restart },
		{ "quit", function() awesome.quit() end },
	}},
	{ "power", {
		{ "off", function() awful.spawn{"poweroff"} end },
		{ "reboot", function() awful.spawn{"reboot"} end }
	}},
}

for i, _ in ipairs(mymainmenu.items) do mymainmenu:delete(i) end
for _, v in ipairs(main_menu) do mymainmenu:add(v) end

-- }}}


-- {{{ Global rules

rules.add_rule{
	rule_any = {
		type = {
			"normal"
		}
	},
	properties = {
		floating = false,
	}
}
rules.add_rule{
	rule_any = {
		role = {
			"Preferences",                           -- ┐
			"AlarmWindow",                           -- │
			"EventDialog",                           -- ├ Thunderbird
			"EventSummaryDialog",                    -- │
			"PasswordManager",                       -- │
			"filterlist",                            -- │
			"view-source",                           -- │
			"News-BlogSubscriptions",                -- ┘
			"Organizer",                             -- ┬ Firefox
			"page-info",                             -- ┘
			"pop-up",
		},
		name = {
			"Enigmail Preferences",
		},
		class = {
			"Gcr-prompter",                          --   Keyring
		},
		type = {
			"dialog",
		},
	},
	properties = {
		floating = true,
	}
}
rules.add_rule{
	rule = {
		class = "Steam",
	},
	except_any = {
		type = { "menu" },
		name = { "^Steam$" },
	},
	properties = {
		floating = true,
		skip_taskbar = true,
	},
}

rules.add_rule{
	rule = {
		class = "org.remmina.Remmina",
		type = "normal",
	},
	except = {
		name = "Remmina Remote Desktop Client",
	},
	callback = function (c)
		c:tags{
			awful.tag.add(c.name, {
				screen = c.screen,
				layout = awful.layout.suit.max.fullscreen,
				selected = true,
				volatile = true,
			}),
		}
	end
}

-- }}}


-- {{{ Global keys
if util.path.has_executable('pactl') then
	local pa_device = PA_DEVICE or '0'
	root.keys(awful.util.table.join(root.keys(),
		util.keys.run({}, 'XF86AudioRaiseVolume', {
			'pactl', 'set-sink-volume', pa_device, '+5%'
		}),
		util.keys.run({}, 'XF86AudioLowerVolume', {
			'pactl', 'set-sink-volume', pa_device, '-5%'
		}),
		util.keys.run({}, 'XF86AudioMute', {
			'pactl', 'set-sink-mute', pa_device, 'toggle'
		})
	))
end
-- }}}


-- {{{ Startup Utilities

awful.spawn.with_line_callback({
	os.getenv('SHELL'), '-c',
	'type systemctl && [ x`systemctl is-system-running` != xoffline ]'
	}, {
		exit = function (reason, code)
			if reason == 'exit' and code == 0 then
				return
			end
			if util.path.has_executable('start-pulseaudio-x11') then
				awful.spawn{'start-pulseaudio-x11'}
			elseif util.path.has_executable('pulseaudio') then
				awful.spawn{'pulseaudio'}
			end
		end
	})

-- }}}
