. ~/.shell/lib/term-setup.sh

SCRIPT_NAME=`basename -- $0`

BOLD=`tput bold || test $? -eq 1`
UNBOLD=`tput sgr0 || test $? -eq 1`

if [ -n "${ZSH_NAME:-}" ]; then
	CURRENT_SHELL=zsh
elif [ -n "${BASH:-}" ]; then
	CURRENT_SHELL=bash
else
	CURRENT_SHELL=sh
fi


die() {
	[ ${CODE:=$?} -gt 1 ] || CODE=2
	echo >&2 "${BOLD}${SCRIPT_NAME}:${UNBOLD} $*"
	exit $CODE
}


has() {
	# TODO: replace local with let/unlet: https://stackoverflow.com/a/18600920
	local type
	type=`_get_type "$1"` || return 1
	case ${2-} in
		is) _match_type "$type" "$3" ;;
		is-not) ! _match_type "$type" "$3" ;;
		'') true ;;
		*) die "Unknown matching argument: $2" ;;
	esac
} #>/dev/null 2>&1


_get_type() {
	# output one of:
	# - <path>
	# - function
	# - builtin
	# - alias
	# - keyword
	# or return 1
	local type
	case $CURRENT_SHELL in
		bash) type=$(type -t "$1") || return 1
			case $type in
				file) type -p "$1" ;;
				*) echo $type ;;
			esac ;;
		zsh) case `whence -w "$1"` in
				*:\ command|*:\ hashed) whence "$1" ;;
				*:\ function) echo function ;;
				*:\ builtin) echo builtin ;;
				*:\ alias) echo alias ;;
				*:\ reserved) echo keyword ;;
				*) exit 1 ;;
			esac ;;
		sh) type=`type "$1"`
			case $type in
				*\ is\ /*|*\ is\ $1) echo "${type##* is }" ;;
				*function) echo function ;;
				*builtin) echo builtin ;;
				*an\ alias\ *) echo alias ;;
				*keyword) echo keyword ;;
				*) exit 1 ;;
			esac ;;
	esac
}


_match_type() {
	case $2 in
		function|builtin|alias|keyword) [ "$1" = "$2" ]; return ;;
		file) [ -x "$1" ]; return ;;
	esac
	case $CURRENT_SHELL in
		bash|zsh) [[ $1 -ef $2 ]]; return ;;
	esac
	[ `expand "$1"` = `expand "$2"` ]
}


basename_filter()
{
	case ${1-} in
		-z) _bf_readargs="-d ''" ;;
	esac
	while read ${_bf_readargs-} name; do
		basename "$name"
	done
	unset _bf_sep
}


case $CURRENT_SHELL in
	zsh) eval '
		splitpaths() {
			local substr
			for substr in ${(s.:.)1}; echo "$substr"
		}
		'
		;;
	bash)
		splitpaths() {
			local IFS=: substr
			for substr in ${1}; do echo "$substr"; done
		}
		;;
esac


if has realpath; then
	expand() { realpath "$1"; }
elif has readlink; then
	expand() { readlink -f "$1"; }
fi

if has date; then
	get_utime() {
		if [ -n "$1" ]; then
			date -r "$1" '+%s'
		else
			date '+%s'
		fi
	}
elif has cscript; then
	get_utime() {
		if [ -n "$1" ]; then
			cscript //nologo ~/.shell/lib/utime.js /from:"$1"
		else
			cscript //nologo ~/.shell/lib/utime.js
		fi
	}
fi
