""
" Toggle terminal buffers from hotkeys
""

function! kodo#toggleterm#Terminal(new_win, ...)
	let term = {}
	let term.origin_buffer = v:null
	let term.buffer = v:null
	let term.window = v:null
	let term.was_new = v:null
	let term.new_win = a:new_win
	let term.closed = v:false

	let cmd = get(a:, 1, v:null)
	let term.cmd = type(cmd) == v:t_list ? cmd :
	\                   cmd  == v:null   ? [ &shell ] :
	\                                      [ &shell, '-c', string(cmd) ]

	" Expand (show) the terminal, optionally in a new window
	"
	" Terminal.expand(window: window-ID)
	" Terminal.expand(new_win: bool = self.new_win)
	"
	" If a new window is requested it is opened by spliting, which obeys the […] 
	" and […] settings.
	function term.expand(...)
		if self.buffer == v:null
			call self.open()
		endif
		if self.window != v:null
			call self.collapse()
		endif

		let window = get(a:, 1, self.new_win)
		if type(window) == v:t_bool
			let new_win = window
		elseif !win_gotoid(window)
			echoerr "Toggle-terminal window not found, ignoring"
			return
		else
			let new_win = v:false
		endif

		let self.origin_buffer = new_win ? v:null : bufnr('')

		if new_win
			silent execute 'sbuffer +startinsert' self.buffer
		else
			silent execute 'buffer +startinsert' self.buffer
		endif

		let self.window = win_getid()
		let self.was_new = new_win

		augroup toggleterm
			au! BufLeave <buffer>
			au  BufLeave <buffer> let b:toggleterm.window = v:null
		augroup END

		if has('nvim') && !has('nvim-0.4')
			" Hack to fix terminal resizing bug
			silent execute 'buffer' bufnr('#')
			redraw
			silent execute 'buffer +startinsert' self.buffer
		endif
	endfunction

	" Collapse (hide) the terminal by restoring any origin buffer if there is 
	" one, else closing the terminal window.
	function term.collapse()
		" silent execute 'au! toggleterm *' self.buffer
		let window = win_getid()
		if !win_gotoid(self.window)
			echoerr "Toggle-terminal window not found, ignoring"
			let self.window = v:null
			return
		elseif bufnr('') != self.buffer
			echoerr "Buffer in toggle-terminal window changed, ignoring"
		else
			call self.on_collapse()
		endif
		if self.window != window
			call win_gotoid(window)
		endif
		let self.window = v:null
	endfunction

	" Collapse the terminal if expanded and vice-versa
	"
	" Terminal.toggle()
	" Terminal.toggle(window: window-ID)
	"
	" If a window is provided, check for the terminal in that window, otherwise 
	" check in the current window. If the terminal is expanded any specified 
	" window will also be passed as the window to expand into.
	function term.toggle(...)
		let window = get(a:, 1, win_getid())
		if self.window == window
			call self.collapse()
		else
			call call(self.expand, a:000)
		endif
	endfunction

	" termopen() on_exit handler
	" Restores any origin buffer and cleans up the terminal buffer
	function term.on_exit(jobid, data, event)
		if self.origin_buffer != v:null
			silent execute 'buffer' self.origin_buffer
		endif
		silent execute 'bdelete!' self.buffer
		call setbufvar(self.buffer, 'toggleterm', v:null)
		let self.buffer = v:null
		let self.window = v:null
		let self.closed = v:true
	endfunction

	" Perform window cleanup on collapse
	" The current buffer is guranteed to be the terminal buffer, and the current 
	" window is guranteed to be the window it was expanded in, otherwise this 
	" hook is not called.
	function term.on_collapse()
		if self.was_new
			call self.close_window()
		else
			call self.restore_buffer()
		endif
	endfunction

	function term.close_window()
		let self.window = v:null
		close
	endfunction

	function term.restore_buffer()
		if self.origin_buffer != v:null
			silent execute 'buffer' self.origin_buffer
		endif
	endfunction

	function term.open()
		" Create the terminal buffer
		let origin = bufnr('')
		enew
		let self.buffer =  bufnr('')

		if self.buffer == origin && origin != 1
			echoerr "New buffer appears to have replaced old buffer?"
		endif

		" Hide from the buffer list
		setl nobuflisted

		" Start the terminal job
		let self.job = termopen(self.cmd, self)

		silent execute 'buffer' origin

		if self.job == -1
			echoerr "Not found or executable:" self.cmd[0]
		elseif self.job <= 0
			throw "Failed to open a new terminal; returned: " + self.job
		endif

		call setbufvar(self.buffer, 'toggleterm', self)
		let self.closed = v:false
	endfunction

	return term
endfunction
