" Dotfiles specific Vim spelling and spell file configuration

" File of spelling 'abbreviations'; auto-corrections in other words
source ~/.vim/spell/auto.vim

let s:spellfiles = [".local.utf8.add"]
let s:spellfiles += glob("~/.vim/spell/*.add", v:false, v:true)
let s:spellfiles += glob("~/.vim/spell/ft/*.add", v:false, v:true)
let &spellfile = join(s:spellfiles, ",")

function s:manage_spell_word(action, spellfile, words)
	let s:spellfiles = uniq(sort(
		\ s:spellfiles +
		\ split(&spellfile, ',') +
		\ [a:spellfile]
		\ ))
	let &spellfile = a:spellfile
	for word in a:words
		execute a:action." ".expand(word)
	endfor
	let &spellfile = join(s:spellfiles, ",")
endfunction

function s:manage_ft_word(action, word)
	if &filetype == ""
		echoerr "No filetype set for this buffer"
		return
	endif
	let spellfile = expand("~/.vim/spell/ft/".&filetype.".utf8.add")
	call s:manage_spell_word(a:action, spellfile, [a:word])
endfunction

function s:manage_classified_word(action, class, ...)
	let spellfile = expand("~/.vim/spell/".a:class.".utf8.add")
	let words = a:000
	if len(words) == 0
		let words = ["<cword>"]
	endif
	call s:manage_spell_word(a:action, spellfile, words)
endfunction

function s:complete(cur, cmd, pos)
	echom a:cmd[:a:pos]
	if a:cmd[:a:pos] =~ " [^ ]+ "
		" Not on the first argument, no suggestions
		return []
	endif
	return map(glob("~/.vim/spell/*.utf8.add", 0, 1),
				\ {key, val -> fnamemodify(val, ":t:r:r")})
endfunction


nnoremap <silent> zLg :call <SID>manage_ft_word("spellgood",  "<cword>")<CR>
nnoremap <silent> zLw :call <SID>manage_ft_word("spellwrong", "<cword>")<CR>
nnoremap <silent> zLu :call <SID>manage_ft_word("spellundo",  "<cword>")<CR>

command -nargs=+ -complete=customlist,s:complete SpellGood  :call s:manage_classified_word("spellgood", <f-args>)
command -nargs=+ -complete=customlist,s:complete SpellWrong :call s:manage_classified_word("spellgood", <f-args>)
command -nargs=+ -complete=customlist,s:complete SpellUndo  :call s:manage_classified_word("spellgood", <f-args>)
